@@ -1,7 +1,7 @@
 /*
     Qalculate (library)
 
-    Copyright (C) 2003-2007, 2008, 2016  Hanna Knutsson (hanna_k@fmgirl.com)
+    Copyright (C) 2003-2007, 2008, 2016  Hanna Knutsson (hanna.knutsson@protonmail.com)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,20 +14,36 @@
 
 #include <libqalculate/includes.h>
 
-#include <cln/cln.h>
+#include <gmp.h>
+#include <mpfr.h>
 
 /** @file */
 
 #define EQUALS_PRECISION_DEFAULT 	-1
-#define EQUALS_PRECISION_LOWEST	-2
+#define EQUALS_PRECISION_LOWEST		-2
 #define EQUALS_PRECISION_HIGHEST	-3
 
+typedef enum {
+	NUMBER_TYPE_RATIONAL,
+	NUMBER_TYPE_FLOAT,
+	NUMBER_TYPE_PLUS_INFINITY,
+	NUMBER_TYPE_MINUS_INFINITY
+} NumberType;
+
+typedef enum {
+	INTEGER_TYPE_NONE,
+	INTEGER_TYPE_SINT,
+	INTEGER_TYPE_UINT,
+	INTEGER_TYPE_ULONG,
+	INTEGER_TYPE_SLONG,
+	INTEGER_TYPE_SIZE
+} IntegerType;
 
 /// A number.
 /**
 * Can be rational, floating point, complex or infinite.
 * Has arbitrary precision (uses Calculator::precision()) and infinitely large rational numbers.
-* Implimented using CLN numbers.
+* Implimented using GNU MP and MPFR.
  */
 class Number {
 	
@@ -35,14 +51,20 @@ class Number {
 
 	protected:
 	
-		void removeFloatZeroPart();
-		void testApproximate();
 		void testInteger();
+		bool testErrors(int error_level = 1) const;
+		bool testFloatResult(bool allow_infinite_result = true, int error_level = 1, bool test_integer = true);
 		void setPrecisionAndApproximateFrom(const Number &o);
 
-		cln::cl_N value;
-		bool b_inf, b_pinf, b_minf;
-		bool b_approx;
+		mpq_t r_value;
+		mpfr_t fu_value;
+		mpfr_t fl_value;
+		
+		Number *i_value;
+		
+		NumberType n_type;
+		
+		bool b_approx, b_imag;
 		int i_precision;
 
 	public:
@@ -65,7 +87,7 @@ class Number {
 		* @param denominator
 		* @param exp_10
  		*/
-		Number(int numerator, int denominator = 1, int exp_10 = 0);
+		Number(long int numerator, long int denominator = 1, long int exp_10 = 0);
 		/**
 		* Constructs a copy of a number.
  		*/
@@ -73,41 +95,71 @@ class Number {
 		virtual ~Number();
 		
 		void set(string number, const ParseOptions &po = default_parse_options);
-		void set(int numerator, int denominator = 1, int exp_10 = 0);
-		void setInfinity();
-		void setPlusInfinity();
-		void setMinusInfinity();
+		void set(long int numerator, long int denominator = 1, long int exp_10 = 0, bool keep_precision = false, bool keep_imag = false);
+		void setPlusInfinity(bool keep_precision = false, bool keep_imag = false);
+		void setMinusInfinity(bool keep_precision = false, bool keep_imag = false);
 		void setFloat(double d_value);
+		bool setInterval(const Number &nr_lower, const Number &nr_upper, bool keep_precision = false);
 
-		void setInternal(const cln::cl_N &cln_value);
+		void setInternal(const mpz_t &mpz_value, bool keep_precision = false, bool keep_imag = false);
+		void setInternal(mpz_srcptr mpz_value, bool keep_precision = false, bool keep_imag = false);
+		void setInternal(const mpq_t &mpq_value, bool keep_precision = false, bool keep_imag = false);
+		void setInternal(const mpz_t &mpz_num, const mpz_t &mpz_den, bool keep_precision = false, bool keep_imag = false);
+		void setInternal(const mpfr_t &mpfr_value, bool merge_precision = false, bool keep_imag = false);
 
 		void setImaginaryPart(const Number &o);
-		void setImaginaryPart(int numerator, int denominator = 1, int exp_10 = 0);
-		void set(const Number &o);
-		void clear();
-
-		const cln::cl_N &internalNumber() const;
+		void setImaginaryPart(long int numerator, long int denominator = 1, long int exp_10 = 0);
+		void set(const Number &o, bool merge_precision = false, bool keep_imag = false);
+		void clear(bool keep_precision = false);
+		void clearReal();
+		void clearImaginary();
+		
+		const mpq_t &internalRational() const;
+		const mpfr_t &internalUpperFloat() const;
+		const mpfr_t &internalLowerFloat() const;
+		mpq_t &internalRational();
+		mpfr_t &internalUpperFloat();
+		mpfr_t &internalLowerFloat();
+		Number *internalImaginary() const;
+		void markAsImaginaryPart(bool is_imag = true);
+		const NumberType &internalType() const;
+		bool setToFloatingPoint();
+		void precisionToInterval();
+		bool intervalToPrecision(long int min_precision = 2);
+		void intervalToMidValue();
+		bool mergeInterval(const Number &o, bool set_to_overlap = false);
+		void setUncertainty(const Number &o, bool force_interval = false);
+		Number uncertainty() const;
+		Number relativeUncertainty() const;
 		
 		double floatValue() const;
 		/**
 		* Converts a number to an integer. If the number does not represent an integer it will rounded using round().
 		*
-		* @param[out] overflow If overflow is non-null it will be set to true if the number was to large to fit in an int.
+		* @param[out] overflow If overflow is non-null it will be set to true if the number was to large to fit the return type.
 		* @return Resulting integer.
  		*/
 		int intValue(bool *overflow = NULL) const;
+		unsigned int uintValue(bool *overflow = NULL) const;
+		long int lintValue(bool *overflow = NULL) const;
+		long long int llintValue() const;
+		unsigned long int ulintValue(bool *overflow = NULL) const;
 		
 		/** Returns true if the number is approximate.
 		*
  		* @return true if the number is approximate.
  		*/
 		bool isApproximate() const;
-		/** Returns true if the number has an approximate representation/is of approximate type -- if it is a floating point number. Numbers of approximate type are always approximate, but the reversed relation is not always true.
+		/** Returns true if the number is a floating point number.
 		*
  		* @return true if the number has an approximate representation.
  		*/
-		bool isApproximateType() const;
-		/** Defines the number as approximate or exact. If a number of approximate type is set as exact, it will be converted to a rational number.
+		bool isFloatingPoint() const;
+		
+		bool isInterval(bool ignore_imag = true) const;
+		bool imaginaryPartIsInterval() const;
+		
+		/** Defines the number as approximate or exact.
 		*
  		* @param is_approximate If the number shall be regarded as approximate.
  		*/
@@ -117,7 +169,7 @@ class Number {
 		*
  		* @return Precision of the number or -1 if the number is exact or the precision has not been set.
  		*/
-		int precision() const;
+		int precision(int calculate_from_interval = 0) const;
 		void setPrecision(int prec);
 		
 		bool isUndefined() const;
@@ -125,22 +177,21 @@ class Number {
 		*
  		* @return true if the number is infinite.
  		*/
-		bool isInfinite() const;
-		/** Returns true if the number is infinity, if the number is plus or minus infinity (which is not known).
-		*
- 		* @return true if the number is infinity.
- 		*/
-		bool isInfinity() const;
+		bool isInfinite(bool ignore_imag = true) const;
 		/** Returns true if the number is plus infinity.
 		*
  		* @return true if the number is plus infinity.
  		*/
-		bool isPlusInfinity() const;
+		bool isPlusInfinity(bool ignore_imag = false) const;
 		/** Returns true if the number is minus infinity.
 		*
  		* @return true if the number is minus infinity.
  		*/
-		bool isMinusInfinity() const;
+		bool isMinusInfinity(bool ignore_imag = false) const;
+
+		bool includesInfinity(bool ignore_imag = false) const;
+		bool includesPlusInfinity() const;
+		bool includesMinusInfinity() const;
 		
 		/** Returns the real part of the number if it is complex, or a copy if it is real.
 		*
@@ -156,8 +207,11 @@ class Number {
 		Number denominator() const;
 		Number complexNumerator() const;
 		Number complexDenominator() const;
+		Number lowerEndPoint(bool include_imag = false) const;
+		Number upperEndPoint(bool include_imag = false) const;
 
 		void operator = (const Number &o);
+		void operator = (long int i);
 		void operator -- (int);
 		void operator ++ (int);
 		Number operator - () const;
@@ -166,6 +220,11 @@ class Number {
 		Number operator + (const Number &o) const;
 		Number operator - (const Number &o) const;
 		Number operator ^ (const Number &o) const;
+		Number operator * (long int i) const;
+		Number operator / (long int i) const;
+		Number operator + (long int i) const;
+		Number operator - (long int i) const;
+		Number operator ^ (long int i) const;
 		Number operator && (const Number &o) const;
 		Number operator || (const Number &o) const;
 		Number operator ! () const;
@@ -175,9 +234,16 @@ class Number {
 		void operator += (const Number &o);
 		void operator -= (const Number &o);
 		void operator ^= (const Number &o);
+		void operator *= (long int i);
+		void operator /= (long int i);
+		void operator += (long int i);
+		void operator -= (long int i);
+		void operator ^= (long int i);
 		
 		bool operator == (const Number &o) const;
 		bool operator != (const Number &o) const;
+		bool operator == (long int i) const;
+		bool operator != (long int i) const;
 		
 		bool bitAnd(const Number &o);
 		bool bitOr(const Number &o);
@@ -191,12 +257,13 @@ class Number {
 		bool hasRealPart() const;
 		bool hasImaginaryPart() const;
 		bool isComplex() const;
-		bool isInteger() const;
+		bool isInteger(IntegerType integer_type = INTEGER_TYPE_NONE) const;
 		Number integer() const;
 		bool isRational() const;
 		bool isReal() const;
 		bool isFraction() const;
 		bool isZero() const;
+		bool isNonZero() const;
 		bool isOne() const;
 		bool isTwo() const;
 		bool isI() const;
@@ -207,15 +274,22 @@ class Number {
 		bool isPositive() const;
 		bool isNonPositive() const;
 		bool realPartIsNegative() const;
+		bool realPartIsNonNegative() const;
 		bool realPartIsPositive() const;
+		bool realPartIsNonZero() const;
+		bool realPartIsRational() const;
 		bool imaginaryPartIsNegative() const;
 		bool imaginaryPartIsPositive() const;
+		bool imaginaryPartIsNonZero() const;
 		bool hasNegativeSign() const;
 		bool hasPositiveSign() const;
 		bool equalsZero() const;
-		bool equals(const Number &o) const;
-		bool equalsApproximately(const Number &o, int prec) const;
-		ComparisonResult compare(const Number &o) const;
+		bool equals(const Number &o, bool allow_interval = false) const;
+		bool equals(long int i) const;
+		int equalsApproximately(const Number &o, int prec) const;
+		ComparisonResult compare(const Number &o, bool ignore_imag = false) const;
+		ComparisonResult compareAbsolute(const Number &o, bool ignore_imag = false) const;
+		ComparisonResult compare(long int i) const;
 		ComparisonResult compareApproximately(const Number &o, int prec = EQUALS_PRECISION_LOWEST) const;
 		ComparisonResult compareImaginaryParts(const Number &o) const;
 		ComparisonResult compareRealParts(const Number &o) const;
@@ -223,7 +297,22 @@ class Number {
 		bool isLessThan(const Number &o) const;
 		bool isGreaterThanOrEqualTo(const Number &o) const;
 		bool isLessThanOrEqualTo(const Number &o) const;
+		bool isGreaterThan(long int i) const;
+		bool isLessThan(long int i) const;
+		bool isGreaterThanOrEqualTo(long int i) const;
+		bool isLessThanOrEqualTo(long int i) const;
+		bool absIsGreaterThan(const Number &o) const;
+		bool absIsLessThan(const Number &o) const;
 		bool isEven() const;
+		bool numeratorIsGreaterThan(long int i) const;
+		bool numeratorIsLessThan(long int i) const;
+		bool numeratorEquals(long int i) const;
+		bool denominatorIsGreaterThan(long int i) const;
+		bool denominatorIsLessThan(long int i) const;
+		bool denominatorEquals(long int i) const;
+		bool denominatorIsGreater(const Number &o) const;
+		bool denominatorIsLess(const Number &o) const;
+		bool denominatorIsEqual(const Number &o) const;
 		bool denominatorIsEven() const;
 		bool denominatorIsTwo() const;
 		bool numeratorIsEven() const;
@@ -239,24 +328,28 @@ class Number {
  		* @return true if the operation was successful.
  		*/
 		bool add(const Number &o);
+		bool add(long int i);
 		/** Subtracts from to the number (x-o).
 		*
 		* @param o Number to subtract.
  		* @return true if the operation was successful.
  		*/
 		bool subtract(const Number &o);
+		bool subtract(long int i);
 		/** Multiply the number (x*o).
 		*
 		* @param o Number to multiply with.
  		* @return true if the operation was successful.
  		*/
 		bool multiply(const Number &o);
+		bool multiply(long int i);
 		/** Divide the number (x/o).
 		*
 		* @param o Number to divide by.
  		* @return true if the operation was successful.
  		*/
 		bool divide(const Number &o);
+		bool divide(long int i);
 		/** Invert the number (1/x).
 		*
  		* @return true if the operation was successful.
@@ -269,6 +362,10 @@ class Number {
  		* @return true if the operation was successful.
  		*/
 		bool raise(const Number &o, bool try_exact = true);
+		bool sqrt();
+		bool cbrt();
+		bool root(const Number &o);
+		bool allroots(const Number &o, vector<Number> &roots);
 		/** Multiply the number with a power of ten (x*10^o).
 		*
 		* @param o Number to raise 10 by.
@@ -310,7 +407,9 @@ class Number {
 		bool ceil(const Number &o);
 		bool trunc(const Number &o);
 		bool mod(const Number &o);
+		bool isIntegerDivisible(const Number &o) const;
 		bool isqrt();
+		bool isPerfectSquare() const;
 		bool round();
 		bool floor();
 		bool ceil();
@@ -322,6 +421,7 @@ class Number {
 		bool irem(const Number &o);
 		bool irem(const Number &o, Number &q);
 		bool iquo(const Number &o);
+		bool iquo(unsigned long int i);
 		bool iquo(const Number &o, Number &r);
 
 		int getBoolean() const;
@@ -332,7 +432,7 @@ class Number {
 		
 		/** Set the number to e, the base of natural logarithm, calculated with the current default precision.
  		*/
-		void e();
+		void e(bool use_cached_number = true);
 		/** Set the number to pi, Archimede's constant, calculated with the current default precision.
  		*/
 		void pi();
@@ -346,7 +446,15 @@ class Number {
 		*
  		* @return true if the calculation was successful.
  		*/
-		bool zeta();			
+		bool zeta();
+		
+		bool gamma();
+		bool digamma();
+		bool airy();
+		bool erf();
+		bool erfc();
+		bool besselj(const Number &o);
+		bool bessely(const Number &o);
 		
 		bool sin();
 		bool asin();
@@ -358,20 +466,34 @@ class Number {
 		bool acosh();
 		bool tan();
 		bool atan();
+		bool atan2(const Number &o, bool allow_zero = false);
+		bool arg();
 		bool tanh();
 		bool atanh();
-		bool ln();	
+		bool ln();
 		bool log(const Number &o);
 		bool exp();
 		bool lambertW();
 		bool gcd(const Number &o);
 		bool lcm(const Number &o);
 		
+		bool polylog(const Number &o);
+		bool igamma(const Number &o);
+		bool expint();
+		bool logint();
+		bool sinint();
+		bool sinhint();
+		bool cosint();
+		bool coshint();
+		
 		bool factorial();
 		bool multiFactorial(const Number &o);
 		bool doubleFactorial();
 		bool binomial(const Number &m, const Number &k);
 		bool factorize(vector<Number> &factors);
+		
+		void rand();
+		void intRand(const Number &ceil);
 	
 		bool add(const Number &o, MathOperation op); 
 
@@ -384,4 +506,6 @@ class Number {
 	
 };
 
+ostream& operator << (ostream &os, const Number&);
+
 #endif
