@@ -641,6 +641,28 @@ bool makeDir(string dirpath) {
 	return mkdir(dirpath.c_str(), S_IRWXU) == 0;
 #endif
 }
+
+bool recursiveMakeDir(string dirpath) {
+#ifdef WIN32
+	return _mkdir(dirpath.c_str()) == 0;
+#else
+	char tmp[256];
+	char *p = NULL;
+	size_t len;
+	snprintf(tmp, sizeof(tmp), "%s", dirpath.c_str());
+	len = strlen(tmp);
+	if(tmp[len - 1] == '/') tmp[len - 1] = 0;
+	for(p = tmp + 1; *p; p++) {
+		if(*p == '/') {
+			*p = 0;
+			if(!dirExists(tmp)) mkdir(tmp, S_IRWXU);
+			*p = '/';
+		}
+	}
+	return mkdir(tmp, S_IRWXU) == 0;
+#endif
+}
+
 bool removeDir(string dirpath) {
 #ifdef WIN32
 	return _rmdir(dirpath.c_str()) == 0;
