@@ -1,23 +1,195 @@
 #include <libqalculate/qalculate.h>
 
-int main(int argc, char *argv[]) {
+void display_errors() {
+	if(!CALCULATOR->message()) return;
+	while(true) {
+		MessageType mtype = CALCULATOR->message()->type();
+		if(mtype == MESSAGE_ERROR) cout << "error: ";
+		else if(mtype == MESSAGE_WARNING) cout << "warning: ";
+		cout << CALCULATOR->message()->message() << endl;
+		if(!CALCULATOR->nextMessage()) break;
+	}
+}
+void clear_errors() {
+	if(!CALCULATOR->message()) return;
+	while(true) {
+		if(!CALCULATOR->nextMessage()) break;
+	}
+}
 
-	new Calculator();
-	CALCULATOR->loadGlobalDefinitions();
-	
-	CALCULATOR->setPrecision(10);
+void test_integration4(const MathStructure &mstruct) {
+	MathStructure x_var(CALCULATOR->v_x);
+	cout << "Integration test: " << mstruct.print() << endl;
+	MathStructure mstruct2(mstruct);
+	EvaluationOptions eo;
+	eo.parse_options.angle_unit = ANGLE_UNIT_RADIANS;
+	eo.assume_denominators_nonzero = true;
+	mstruct2.integrate(x_var, eo, true, 1);
+	mstruct2.eval(eo);
+	if(mstruct2.containsFunction(CALCULATOR->f_integrate)) {clear_errors(); return;}
+	mstruct2.differentiate(x_var, eo);
+	MathStructure mstruct3(mstruct2);
+	mstruct3.eval(eo);
+	mstruct3.replace(x_var, 3);
+	mstruct3.eval(eo);
+	display_errors();
+	string str1 = mstruct3.print();
+	cout << str1 << endl;
+	mstruct3 = mstruct;
+	mstruct3.replace(x_var, 3);
+	mstruct3.eval(eo);
+	display_errors();
+	string str2 = mstruct3.print();
+	cout << str2 << endl;
+	if(str1 != str2) cout << "!!!" << endl;
+	mstruct3 = mstruct2;
+	mstruct3.replace(x_var, -5);
+	mstruct3.eval(eo);
+	display_errors();
+	str1 = mstruct3.print();
+	cout << str1 << endl;
+	mstruct3 = mstruct;
+	mstruct3.replace(x_var, -5);
+	mstruct3.eval(eo);
+	display_errors();
+	str2 = mstruct3.print();
+	cout << str2 << endl;
+	if(str1 != str2) cout << "!!!" << endl;
+	cout << "________________________________________________" << endl;
+}
+void test_integration3(const MathStructure &mstruct, const MathStructure &mstruct_arg) {
+	MathStructure mstruct2(mstruct);
+	test_integration4(mstruct2);
+	mstruct2 = mstruct;
+	mstruct2 *= CALCULATOR->v_x;
+	test_integration4(mstruct2);
+	mstruct2.last() ^= nr_two;
+	test_integration4(mstruct2);
+	mstruct2.last()[1] = nr_minus_one;
+	test_integration4(mstruct2);
+	mstruct2 = mstruct;
+	mstruct2 *= mstruct_arg;
+	test_integration4(mstruct2);
+	mstruct2 = mstruct;
+	mstruct2 *= mstruct;
+	test_integration4(mstruct2);
+}
+void test_integration2(const MathStructure &mstruct) {
+	MathStructure mstruct2(mstruct);
+	mstruct2.transform(CALCULATOR->f_ln);
+	test_integration3(mstruct2, mstruct);
+	mstruct2 = mstruct;
+	mstruct2 *= CALCULATOR->getRadUnit();
+	mstruct2.transform(CALCULATOR->f_sin);
+	test_integration3(mstruct2, mstruct);
+	mstruct2 = mstruct;
+	mstruct2 *= CALCULATOR->getRadUnit();
+	mstruct2.transform(CALCULATOR->f_cos);
+	test_integration3(mstruct2, mstruct);
+	mstruct2 = mstruct;
+	mstruct2 *= CALCULATOR->getRadUnit();
+	mstruct2.transform(CALCULATOR->f_tan);
+	test_integration3(mstruct2, mstruct);
+	mstruct2 = mstruct;
+	mstruct2.transform(CALCULATOR->f_asin);
+	test_integration3(mstruct2, mstruct);
+	mstruct2 = mstruct;
+	mstruct2.transform(CALCULATOR->f_acos);
+	test_integration3(mstruct2, mstruct);
+	mstruct2 = mstruct;
+	mstruct2.transform(CALCULATOR->f_atan);
+	test_integration3(mstruct2, mstruct);
+	mstruct2 = mstruct;
+	mstruct2.transform(CALCULATOR->f_sinh);
+	test_integration3(mstruct2, mstruct);
+	mstruct2 = mstruct;
+	mstruct2.transform(CALCULATOR->f_cosh);
+	test_integration3(mstruct2, mstruct);
+	mstruct2 = mstruct;
+	mstruct2.transform(CALCULATOR->f_tanh);
+	test_integration3(mstruct2, mstruct);
+	mstruct2 = mstruct;
+	mstruct2.transform(CALCULATOR->f_asinh);
+	test_integration3(mstruct2, mstruct);
+	mstruct2 = mstruct;
+	mstruct2.transform(CALCULATOR->f_acosh);
+	test_integration3(mstruct2, mstruct);
+	mstruct2 = mstruct;
+	mstruct2.transform(CALCULATOR->f_atanh);
+	test_integration3(mstruct2, mstruct);
+	mstruct2 = mstruct;
+	mstruct2 ^= nr_two;
+	test_integration3(mstruct2, mstruct);
+	mstruct2 = mstruct;
+	mstruct2 ^= nr_minus_one;
+	test_integration3(mstruct2, mstruct);
+	mstruct2 = mstruct;
+	mstruct2 ^= Number(-2, 1);
+	test_integration3(mstruct2, mstruct);
+	mstruct2 = mstruct;
+	mstruct2 ^= Number(3, 1);
+	test_integration3(mstruct2, mstruct);
+	mstruct2 = mstruct;
+	mstruct2 ^= Number(-3, 1);
+	test_integration3(mstruct2, mstruct);
+	mstruct2 = mstruct;
+	mstruct2 ^= Number(1, 3);
+	test_integration3(mstruct2, mstruct);
+}
+void test_integration() {
+	MathStructure mstruct;
+	CALCULATOR->parse(&mstruct, "4x+5");
+	test_integration2(mstruct);
+	CALCULATOR->parse(&mstruct, "-2x+7");
+	test_integration2(mstruct);
+	CALCULATOR->parse(&mstruct, "4.7x-5.2");
+	test_integration2(mstruct);
+	CALCULATOR->parse(&mstruct, "-4.3x-5");
+	test_integration2(mstruct);
+	CALCULATOR->parse(&mstruct, "4x");
+	test_integration2(mstruct);
+	CALCULATOR->parse(&mstruct, "-2.3x");
+	test_integration2(mstruct);
+	CALCULATOR->parse(&mstruct, "x+6");
+	test_integration2(mstruct);
+	CALCULATOR->parse(&mstruct, "x-7");
+	test_integration2(mstruct);
+	CALCULATOR->parse(&mstruct, "x");
+	test_integration2(mstruct);
+	CALCULATOR->parse(&mstruct, "x^2");
+	test_integration2(mstruct);
+	CALCULATOR->parse(&mstruct, "2x^2+5");
+	test_integration2(mstruct);
+	CALCULATOR->parse(&mstruct, "-2x^2-5");
+	test_integration2(mstruct);
+	CALCULATOR->parse(&mstruct, "sqrt(x)");
+	test_integration2(mstruct);
+	CALCULATOR->parse(&mstruct, "sqrt(3x+3)");
+	test_integration2(mstruct);
+	CALCULATOR->parse(&mstruct, "5*sqrt(3x)-2");
+	test_integration2(mstruct);
+	CALCULATOR->parse(&mstruct, "cbrt(3x+3)");
+	test_integration2(mstruct);
+	CALCULATOR->parse(&mstruct, "(3x+3)^(1/3)");
+	test_integration2(mstruct);
+	CALCULATOR->parse(&mstruct, "cbrt(x)");
+	test_integration2(mstruct);
+	CALCULATOR->parse(&mstruct, "x^(1/3)");
+	test_integration2(mstruct);
+	CALCULATOR->parse(&mstruct, "5^x");
+	test_integration2(mstruct);
+}
 
-#define TEST_INTERVALS
+void test_intervals(bool use_interval) {
 
-	CALCULATOR->useIntervalArithmetic(true);
+	CALCULATOR->useIntervalArithmetic(use_interval);
 	PrintOptions po;
 	//po.interval_display = INTERVAL_DISPLAY_SIGNIFICANT_DIGITS;
 	
 	vector<Number> nrs;
-#ifdef TEST_INTERVALS
+
 	nrs.push_back(nr_plus_inf);
 	nrs.push_back(nr_minus_inf);
-#endif
 	nrs.push_back(nr_zero);
 	nrs.push_back(nr_half);
 	nrs.push_back(nr_minus_half);
@@ -25,7 +197,7 @@ int main(int argc, char *argv[]) {
 	nrs.push_back(nr_minus_one);
 	nrs.push_back(nr_two);
 	Number nr;
-#ifdef TEST_INTERVALS
+
 #define INCLUDES_INFINITY(x) (x.includesInfinity())
 #define IS_INTERVAL(x) (x.isInterval(false))
 	nr.setInterval(nr_minus_half, nr_half); nrs.push_back(nr); 
@@ -54,10 +226,6 @@ int main(int argc, char *argv[]) {
 	nr.setInterval(nr_plus_inf, nr_minus_one); nrs.push_back(nr);
 	nr.setInterval(nr_plus_inf, nr_one); nrs.push_back(nr);
 	nr.setImaginaryPart(nrs[nrs.size() - 2]); nrs.push_back(nr); nr.setImaginaryPart(nr_one); nrs.push_back(nr); nr.setImaginaryPart(nr_minus_inf); nrs.push_back(nr);
-#else
-#define INCLUDES_INFINITY(x) (false)
-#define IS_INTERVAL(x) (false)
-#endif
 	nr.setFloat(0.5); nrs.push_back(nr);
 	nr.setFloat(-0.5); nrs.push_back(nr);
 	nr.setImaginaryPart(nrs[nrs.size() - 2]); nr.setImaginaryPart(nr_one); nrs.push_back(nr);
@@ -440,7 +608,6 @@ int main(int argc, char *argv[]) {
 		else cout << "FAILED" << endl;
 			if(!INCLUDES_INFINITY(nr)) nrsum3 += nr;
 	}
-	return 0;
 	
 	/*Number nr1, nr2, nr3, nr4, nr5;
 	nr1.setInterval(Number(1, 1), Number(2, 1));
@@ -660,8 +827,209 @@ int main(int argc, char *argv[]) {
 	nrm = nr4; nrm.divide(-2);
 	cout << "4: " << nrm.print() << endl;
 	nrm = nr5; nrm.add(-2);
-	cout << "5: " << nrm.print() << endl;
-	
-	return 0;*/
+	cout << "5: " << nrm.print() << endl;*/
 
 }
+
+void speed_test() {
+		
+	/*UserFunction f1("", "", "x^2/3");
+	UserFunction f2("", "", "1/x");
+	UserFunction f3("", "", "x/4-3");
+	UserFunction f4("", "", "sin(x rad)");
+	
+	MathStructure v;
+	v.clearVector();
+	MathStructure m1 = f1.calculate*/
+	
+	//Number nr(1);
+	//Number nr_change(1, 8000);
+	/*for(size_t i = 0; i < 800000; i++) {
+		Number n(nr);
+		n ^= 2;
+		n /= 3;
+		nr += nr_change;
+	}
+	nr.set(1);
+	for(size_t i = 0; i < 800000; i++) {
+		Number n(nr);
+		n.recip();
+		nr += nr_change;
+	}
+	nr.set(1);
+	for(size_t i = 0; i < 800000; i++) {
+		Number n(nr);
+		n /= 4;
+		n -= 3;
+		nr += nr_change;
+	}*/
+	/*nr.set(1);
+	for(size_t i = 0; i < 800000; i++) {
+		Number n(nr);
+		n.sin();
+		n *= 2;
+		n -= nr;
+		n.cos();
+		nr += nr_change;
+	}*/
+	
+	/*mpfr_set_default_prec(32);
+	mpfr_t nr;
+	mpfr_init(nr);
+	mpfr_set_ui(nr, 1, MPFR_RNDN);
+	mpfr_t nr_change;
+	mpfr_init(nr_change);
+	mpfr_set_ui(nr_change, 1, MPFR_RNDN);
+	mpfr_div_ui(nr_change, nr_change, 8000, MPFR_RNDN);
+	mpfr_t n;
+	mpfr_init(n);*/
+	/*for(size_t i = 0; i < 800000; i++) {
+		mpfr_pow_ui(n, nr, 2, MPFR_RNDN);
+		mpfr_div_ui(n, n, 3, MPFR_RNDN);
+		mpfr_add(nr, nr, nr_change, MPFR_RNDN);
+	}
+	mpfr_set_ui(nr, 1, MPFR_RNDN);
+	for(size_t i = 0; i < 800000; i++) {
+		mpfr_ui_div(n, 1, nr, MPFR_RNDN);
+		mpfr_add(nr, nr, nr_change, MPFR_RNDN);
+	}
+	mpfr_set_ui(nr, 1, MPFR_RNDN);
+	for(size_t i = 0; i < 800000; i++) {
+		mpfr_div_ui(n, nr, 4, MPFR_RNDN);
+		mpfr_sub_ui(n, n, 3, MPFR_RNDN);
+		mpfr_add(nr, nr, nr_change, MPFR_RNDN);
+	}*/
+	/*mpfr_set_ui(nr, 1, MPFR_RNDN);
+	for(size_t i = 0; i < 800000; i++) {
+		mpfr_sin(n, nr, MPFR_RNDN);
+		mpfr_mul_ui(n, nr, 2, MPFR_RNDN);
+		mpfr_sub(n, n, nr, MPFR_RNDN);
+		mpfr_cos(n, n, MPFR_RNDN);
+		mpfr_add(nr, nr, nr_change, MPFR_RNDN);
+	}*/
+	
+	/*double nr = 1.0;
+	double change = 1.0 / 8000;
+	double n;
+	for(size_t i = 0; i < 800000; i++) {
+		n = pow(nr, 2);
+		n /= 3;
+		nr += change;
+	}
+	nr = 1.0;
+	for(size_t i = 0; i < 800000; i++) {
+		n = 1.0 / nr;
+		nr += change;
+	}
+	nr = 1.0;
+	for(size_t i = 0; i < 800000; i++) {
+		n = nr / 4.0;
+		n -= 3.0;
+		nr += change;
+	}
+	nr = 1.0;
+	for(size_t i = 0; i < 800000; i++) {
+		n = sin(nr);
+		nr += change;
+	}*/
+
+	EvaluationOptions eo;
+	eo.parse_options.angle_unit = ANGLE_UNIT_RADIANS;
+	eo.approximation = APPROXIMATION_APPROXIMATE;
+	//eo.sync_units = false;
+	
+	/*Number n1;
+	for(size_t i = 0; i < 4000000; i++) {
+		n1 += nr_one;
+	}*/
+	
+	/*MathStructure m1;
+	for(size_t i = 0; i < 1000000; i++) {
+		m1.calculateAdd(m_one, eo);
+	}*/
+	
+	/*for(size_t i = 0; i < 2000000; i++) {
+		MathStructure *m1 = new MathStructure();
+	}*/
+	
+	/*for(size_t i = 0; i < 2000000; i++) {
+		Number *m1 = new Number();
+	}*/
+	
+	/*MathStructure m = CALCULATOR->parse("x^2/2+sin(x+3)^2", eo.parse_options);
+	m.eval(eo);
+	MathStructure mx(CALCULATOR->v_x);
+	Number nr;
+	nr.setFloat(4.0);
+	MathStructure mnr(nr);
+	for(size_t i = 0; i < 1000000; i++) {
+		MathStructure r(m);
+		r.replace(mx, mnr);
+		r.eval(eo);
+	}*/
+	/*MathStructure m = CALCULATOR->parse("sin('y')+'y'+'y'+'x'^2/3+'x'+'y'", eo.parse_options);
+	MathStructure mx("x");
+	MathStructure my("y");
+	for(size_t i = 0; i < 500000; i++) {
+		MathStructure m2(m);
+		//m2.replace(mx, m_one);
+		//m2.replace(my, m_one);
+		m2.replace(mx, m_one, my, m_one);
+	}*/
+	/*MathStructure m = CALCULATOR->parse("x^2/3", eo.parse_options);
+	eo.expand = false;
+	m.eval(eo);
+	MathStructure v1 = m.generateVector(CALCULATOR->v_x, 1, 100, 800000, NULL, eo);
+	m = CALCULATOR->parse("1/x");
+	m.eval(eo);
+	MathStructure v2 = m.generateVector(CALCULATOR->v_x, 1, 100, 800000, NULL, eo);
+	m = CALCULATOR->parse("x/4-3");
+	m.eval(eo);
+	MathStructure v3 = m.generateVector(CALCULATOR->v_x, 1, 100, 800000, NULL, eo);
+	m = CALCULATOR->parse("sin(x)", eo.parse_options);
+	m.eval(eo);
+	MathStructure v4 = m.generateVector(CALCULATOR->v_x, 1, 100, 800000, NULL, eo);*/
+	//cout << v4.size() << ":" << v4[0] << ":" << v4[v4.size() - 1] << endl;
+}
+
+int main(int argc, char *argv[]) {
+
+	new Calculator();
+	CALCULATOR->loadGlobalDefinitions();
+	CALCULATOR->loadLocalDefinitions();
+	
+	EvaluationOptions evalops;
+	/*evalops.approximation = APPROXIMATION_TRY_EXACT;
+	evalops.sync_units = true;
+	evalops.structuring = STRUCTURING_SIMPLIFY;
+	evalops.parse_options.unknowns_enabled = false;
+	evalops.parse_options.read_precision = DONT_READ_PRECISION;*/
+	/*evalops.parse_options.base = BASE_DECIMAL;
+	evalops.allow_complex = true;
+	evalops.allow_infinite = true;*/
+	//evalops.auto_post_conversion = POST_CONVERSION_OPTIMAL;
+	evalops.assume_denominators_nonzero = true;
+	//evalops.warn_about_denominators_assumed_nonzero = true;
+	//evalops.parse_options.limit_implicit_multiplication = false;
+	//evalops.parse_options.parsing_mode = PARSING_MODE_ADAPTIVE;
+	evalops.parse_options.angle_unit = ANGLE_UNIT_RADIANS;
+	/*evalops.parse_options.dot_as_separator = CALCULATOR->default_dot_as_separator;
+	evalops.parse_options.comma_as_separator = false;
+	evalops.mixed_units_conversion = MIXED_UNITS_CONVERSION_DEFAULT;*/
+	
+	/*MathStructure mstruct = CALCULATOR->calculate("atanh(2x^2+5)*x^2", evalops);
+	cout << mstruct.integrate(CALCULATOR->v_x, evalops) << endl;
+	mstruct.eval(evalops);
+	cout << mstruct << endl;*/
+	/*mstruct = CALCULATOR->calculate("atanh(3x-2)*x", evalops);
+	mstruct.integrate(CALCULATOR->v_x, evalops);
+	mstruct.eval(evalops);
+	cout << mstruct << endl;*/
+	//speed_test();
+	//test_integration();
+	//test_intervals(true);
+
+	return 0;
+
+}
+
